/**
******************************************************************************
* @file    main.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.0
* @copyright (c) 2018 _KEMET, Ltd
* @brief  IRPyro messaging application to communicate with the IRPyro evaluation tool
*
* @verbatim
 ===============================================================================
                      ##### Description #####
 ===============================================================================
  [..]
     Main function of IRPyro evaluation tool firmware
     - Initialize micro-controller and board
     - Detect sensors present
     - Configure sensors
     - Stream sampling data via UART to the IRPyro evaluation tool GUI
  [..]
 @endverbatim
  ******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>
   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
typedef enum
{
    eevt_initialize,
    command_decode,
    sampling_data_get,
    apply_parameters,
    contact_gui,
    emitter_control,
}
MainState_type;
uint8_t mode_sleeping ;                // capture
uint8_t mode_lowPower ;                // capture
uint8_t UnitID[2]      = {0xda,0xde};
uint8_t BoardType      = 0x47;
uint8_t DarkPixelState = 0x00;
uint8_t FailureFlags   = 0x00;  // coded test result
/**
* bit 0 : external emitter in use
* bit 1 : internal emitter in use
* bit 2 : internal emitter currently ON
* bit 3 : internal emitter in use but permanently OFF
* bit 7 : 0 = normal1 state, = status changed by external command */
uint8_t  EmitterStatus  = 0 ;   //
uint16_t EmitterONTime  = 50 ; //
uint16_t EmitterOFFTime = 50 ; // this means 10Hz with 1:1 mark/space by default
// uint16_t EmitterTimer   = 0;          //

uint8_t isStreaming                 = 0;
uint8_t isStatus                    = 1;
uint8_t status_need_to_send         = 1;
uint8_t error_flag                  = 0;
uint8_t num_of_IRPyro_on_demo_board = NUMBER_IRPyro;   // number of devices on a demo board

IRPyro_devices  IRPyro_device      = {0};  // IRPyro sensors main structure
IRPyro_cmd_type IRPyro_device_cmd  = {0};  //
IRPyro_arg_type IRPyro_device_arg  = {0};  //
uint8_t ChannelConfiguration[NUMBER_OF_SCOPES_IN_THE_GUI] = {0,0x03,0x08,0x0D,0x12,0,0} ; // order of the package for the scope on the GUI

static MainState_type state = eevt_initialize;
static void eevt(MainState_type state);
static void fn_sampling_data_get(void);
static void fn_apply_parameters(void);

EXTernal_callback pointer_to_read_function   = fn_sampling_data_get;
EXTernal_callback pointer_to_wakeup_function = fn_apply_parameters;
TIM_callback      pointer_to_status_function = NULL;
/** @defgroup eevt_main IRPyro messaging application
  * @brief main and support functions for state machine
  * @{
  */
/** @defgroup eevt main IRPyro messaging application
  * @{
  */
/**
  * @brief point of entry for the evaluation tool software
  */
int main(void)
{
    while (1)
    {
        eevt(state);
    }
}
/**
* @brief sensor array population
*/
static void sensor_configuration(IRPyro_devices this_list)
{
    /*--------------------------- INITIALIZE CONFIGURATION -------------------*/
    const AFE_reg_type IRPyro_AFE_Master =
    {
        .S7_S0   = 0,                  // Sets the sampling rate to 1,000 samples per second.
        .LP      = LOW_POWER_DISABLE,  // Low power sets the sampling rate to 166 samples per second.
        .HP      = HP_DISABLE,         // High pass filter enabled
        .C_LP    = LOW_PASS_180,       // Low pass filter set at 180Hz
        .CLK_OUT = CLK_ENABLE,         // Clock output enabled to synchronize other IRPyro sensors
        .SYNC    = MASTER,             // IRPyro set as Master issues the synchronization to SLAVE
        .TEMP    = TEMP_DISABLE,       //
        .INT     = INT_ENABLE          // IRPyro indicates data ready enable
    };
    const AFE_reg_type IRPyro_AFE_Slave =
    {
        .S7_S0   = 0,                  // Sets the sampling rate to 1,000 samples per second.
        .LP      = LOW_POWER_DISABLE,  // Low power sets the sampling rate to 166 samples per second.
        .HP      = HP_DISABLE,         // High pass filter enabled
        .C_LP    = LOW_PASS_180,       // Low pass filter set at 180Hz
        .CLK_OUT = CLK_DISABLE,        // Clock output enabled to synchronize other IRPyro sensors
        .SYNC    = SLAVE,              // IRPyro set as Slave receiver synchronization from Master
        .TEMP    = TEMP_DISABLE,       //
        .INT     = INT_ENABLE          // IRPyro indicates data ready enable
    };

    const CCP_reg_type IRPyro_Ch_Cfg =
    {
        .Feedback_cap              = FEEDBACK_200_80, // Feedback capacitance
        .High_pass_filter_setting  = HIGH_PASS_1HZ,   // High pass filter set at 1Hz
        .Feedback_transconductance = TCONDUCT_0_15T,  // Time constant selection
        .status_bit                = CH_ENABLE        // Channel ready to collect data
    };

    const WUP_reg_type IRPyro_WUP_Cfg =
    {
        .UHT          = MAX_WUP_THRESHOLD,            // Upper High Threshold = 255 events
        .ULT          = SAFE_WUP_THRESHOLD,           // Upper Low  Threshold = 180 events
        .LHT          = MIN_WUP_THRESHOLD,            // Low   High Threshold =   0 events
        .LLT          = MIN_WUP_THRESHOLD,            // Low   Low  Threshold =   0 events
        .WT_Threshold = DEFAULT_NO_EVENTS,            // Wake up time Threshold = 10 events
        .DP0_DP2      = NOT_IN_USE_VALUE,             // Active pixel channel
        .CH0_CH2      = SEL_CH_2,                     // Reference channel
        .ST           = SLEEP_MODE_ONE_CH             // Sleep mode type
    };
    /*-----------------    ADDRESS ASSIGNMENT ---------------------------------*/
    /* Assuming four sensors on an IRPyro Break out board or demo board*/
    this_list[0].CS_Port  = 2;
    this_list[0].CS_Pin   = 1;
    this_list[0].INT_Port = 1;
    this_list[0].INT_Pin  = 4;
    this_list[0].address  = 0x70;

    this_list[1].CS_Port  = 2;
    this_list[1].CS_Pin   = 5;
    this_list[1].INT_Port = 1;
    this_list[1].INT_Pin  = 5;
    this_list[1].address  = 0x69;

    this_list[2].CS_Port  = 2;
    this_list[2].CS_Pin   = 4;
    this_list[2].INT_Port = 1;
    this_list[2].INT_Pin  = 6;
    this_list[2].address  = 0x68;

    this_list[3].CS_Port  = 2;
    this_list[3].CS_Pin   = 3;
    this_list[3].INT_Port = 1;
    this_list[3].INT_Pin  = 7;
    this_list[3].address  = 0x67;

    /*----------------- COMMIT CONFIGURATION   ------------*/
    this_list[0].number_of_active_channels = 1;
    this_list[0].AFE_register    = IRPyro_AFE_Master; // MASTER, INT, CLK ENABLED
    this_list[0].CCP_register[2] = IRPyro_Ch_Cfg;
    this_list[0].WUP_register    = IRPyro_WUP_Cfg;

    this_list[1].number_of_active_channels = 1;
    this_list[1].AFE_register    = IRPyro_AFE_Slave; // SLAVE, INT, CLK DISABLED
    this_list[1].CCP_register[2] = IRPyro_Ch_Cfg;
    this_list[1].WUP_register    = IRPyro_WUP_Cfg;

    this_list[2].number_of_active_channels = 1;
    this_list[2].AFE_register    = IRPyro_AFE_Slave; // SLAVE, INT, CLK DISABLED
    this_list[2].CCP_register[2] = IRPyro_Ch_Cfg;
    this_list[2].WUP_register    = IRPyro_WUP_Cfg;

    this_list[3].number_of_active_channels = 1;
    this_list[3].AFE_register    = IRPyro_AFE_Slave; // SLAVE, INT, CLK DISABLED
    this_list[3].CCP_register[2] = IRPyro_Ch_Cfg;
    this_list[3].WUP_register    = IRPyro_WUP_Cfg;
}
/**
 * @brief Detect IRPyro sensors present in the demo board
 * @param this_list IRPyro sensor array
 * @param list_size number of sensors in the array
*/
static void IRPyro_sensor_population(IRPyro_devices this_list, uint8_t * list_size)
{
    uint8_t num_of_IRPyro = *list_size;
    uint8_t devices_present =0;
    const uint8_t types[]= {0x00, 0x7F, 0x77, 0x67, 0x47};

    for(uint8_t current=0 ; current < num_of_IRPyro; ++ current)
    {
        if (IRPyro_open(&this_list[current]) == 1)
        {
            ++ devices_present;
        }
    }
    *list_size = devices_present;
    BoardType = types[devices_present];
    uint8_t calc01 = (uint8_t)( GPIO_ReadInputData( GPIOA ) >> 8  ) & 1 ;
    uint8_t calc02 = (uint8_t)( GPIO_ReadInputData( GPIOA ) >> 10 ) & 6 ;
    uint8_t calc03 = (uint8_t)( GPIO_ReadInputData( GPIOA ) >> 2  ) & 0x38;
    BoardType =   calc01;
    BoardType |=  calc02;
    BoardType |=  calc03;
    BoardType |= 0x40 ;
    if ( devices_present > 1 )
    {
        BoardType &= 0xf7 ;
    }
    if ( devices_present > 2 )
    {
        BoardType &= 0xef ;
    }
    if ( devices_present > 3 )
    {
        BoardType &= 0xdf ;
    }
}
/**
 * @brief Changes the state of all the CS lines as used on the IRPyro backplane board
 * @param state 0 off / 1 on
 */
static void demo_board_sensor_control(int state)
{
    CS_pin_set(2,1,state);
    CS_pin_set(2,3,state);
    CS_pin_set(2,4,state);
    CS_pin_set(2,5,state);
}
/**
 * @brief power cycles the sensors on the board to assure stability
*/
static void sensor_power_stabilize()
{
    // toggle all CS lines
    demo_board_sensor_control(0);
    Micro_controller_delay_ms(30);
    demo_board_sensor_control(1);
    Micro_controller_delay_ms(30);
    demo_board_sensor_control(0);
}
/**
 * @brief assigns read and status functions interrupts
*/
static void callback_setup()
{
    queue_handle_enable();
    EXTernal_Callback_Setup(pointer_to_read_function);
    TIM_Callback_Setup(pointer_to_status_function);
}
/**
 * @brief Initializes board and populates sensors
*/
static void fn_eevt_initialize(void)
{
    uint8_t LED_ready = 0;
    /*-------------------------- board setup ---------------------------------*/
    Micro_controller_template_init();
    sensor_power_stabilize();
    sensor_configuration(IRPyro_device);
    /*-------- test the open_list, read_list and close_list functions --------*/
    IRPyro_sensor_population(IRPyro_device, &num_of_IRPyro_on_demo_board);
    /*-------------------------- open_list   ---------------------------------*/
    LED_ready = IRPyro_open_devices(IRPyro_device, num_of_IRPyro_on_demo_board);
    LED_READY(LED_ready);
    Micro_controller_timeout_set(SEND_STATUS_TIME);

    callback_setup();
    state = command_decode;
}
/**
* @brief decodes a command from the USART.
*/
uint8_t cmd_decoded = 0;
uint8_t sensor_destination = 0;
static void fn_command_decode(void)
{
    uint8_t decode_result = 0;
    const MainState_type next_state[]=
    {
        apply_parameters,
        contact_gui
    };
    decode_result = message_interpreter(&cmd_decoded, &sensor_destination);
    state = next_state[decode_result];
}
/**
 * @brief Sends collected data from configured sensors
*/
static void fn_sampling_data_get(void)
{
    if (isStreaming)
    {
        IRPyro_read_devices(IRPyro_device, num_of_IRPyro_on_demo_board);
    }
}
/**
* @brief changes parameters from one or more sensors
*/
static void fn_apply_parameters(void)
{
    mode_lowPower = IRPyro_device[0].AFE_register.LP;
    IRPyro_ioctl_devices(IRPyro_device, num_of_IRPyro_on_demo_board, &IRPyro_device_cmd, IRPyro_device_arg);
    state = contact_gui;
}
/**
* @brief changes parameters from one or more sensors
*/
static void fn_contact_gui(void)
{
    message_generator(cmd_decoded, sensor_destination);
    state = command_decode;
}
/**
* @brief configures and control the emitter
*/
static void fn_emitter_control(void)
{
    Emitter_control(&EmitterStatus, EmitterONTime, EmitterOFFTime);
}
/**
 * @brief IRPyro evaluation tool state machine
 * @param state indicates next state in the machine
 */
static void eevt(MainState_type state)
{
    static void (* const pf[6])(void) =
    {
        fn_eevt_initialize,
        fn_command_decode,
        fn_sampling_data_get,
        fn_apply_parameters,
        fn_contact_gui,
        fn_emitter_control,
    };
    (*pf[state])();
    (*pf[sampling_data_get])();
    (*pf[emitter_control])();
}
/** end of eevt
  * @}
  */
/** end of eevt_main
  * @}
  */
/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
